@@ -53,7 +53,7 @@ void ilmi_loop(int fd, int itf);
 
 void usage(char *name)
 {
-  fprintf(stderr, "usage: %s [-b] [-d] <interface>\n", name);
+  fprintf(stderr, "usage: %s [-b] [-d] [-l logfile] <interface>\n", name);
   exit(1);
 }
 
@@ -241,7 +241,7 @@ int main(int argc, char *argv[])
 
   set_application("ilmid");
   set_verbosity(NULL, DIAG_INFO);
-  while((opt = getopt(argc, argv, "bdh")) != EOF)
+  while((opt = getopt(argc, argv, "bdhl:")) != EOF)
     switch (opt)
       {
       case 'd':
@@ -250,6 +250,9 @@ int main(int argc, char *argv[])
       case 'b':
 	bg = 1;
 	break;
+      case 'l':
+	set_logfile(optarg);
+	break;
       case 'h':
       default:
 	usage(argv[0]);
