@@ -1,6 +1,6 @@
 /* first.c - Phase I, input data preprocessing */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -17,6 +17,7 @@ int offset = 0;
 int varlen_fields = 0;
 
 static unsigned char mask = 0;
+static int seq = 0;
 
 
 #define ALLOC(field) ({ \
@@ -47,6 +48,7 @@ static void process(FIELD *start)
 
     for (walk = start; walk; walk = walk->next) {
 	/* set field number */
+	walk->seq = seq++;
 	if (*walk->id != '_') {
 	    to_h("#define QF_%s %d\n",walk->id,field);
 	    to_test("    \"%s\",\n",walk->id);
