@@ -35,9 +35,8 @@ static int send_setup(SOCKET *sock)
     Q_DSC dsc;
     int error,size;
 
-    error = 0;
     do {
-	if (++call_ref == 0x800000) call_ref = 1;
+	if (++call_ref == 0x7fffff) call_ref = 1;
 	for (walk = sockets; walk; walk = walk->next)
 	    if (walk->call_ref == call_ref) break;
     }
@@ -52,7 +51,7 @@ static int send_setup(SOCKET *sock)
 				     this */
 #endif
     q_assign(&dsc,QF_sscs_type,0); /* unspecified - LANE wants this */
-    sap_encode(&dsc,sock->remote);
+    error = sap_encode(&dsc,sock->remote);
     q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
     q_assign(&dsc,QF_traf_type,0); /* force presence - UNI 3.0 wants this */
     q_assign(&dsc,QF_upcc,0); /* p2p */
@@ -67,7 +66,9 @@ static int send_setup(SOCKET *sock)
     else if (local_addr[0].state == ls_same && *local_addr[0].addr.sas_addr.prv)
 	    local = &local_addr[0].addr;
     /* @@@ handle E.164 too */
-    if (!local) error = -EADDRNOTAVAIL;
+    if (!local) {
+	if (!error) error = -EADDRNOTAVAIL;
+    }
     else {
 	q_assign(&dsc,QF_cgpn_plan,2);
 	q_assign(&dsc,QF_cgpn_type,0);
@@ -76,6 +77,7 @@ static int send_setup(SOCKET *sock)
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
 	q_assign(&dsc,QF_vci,assign_vci); /* @@@ */
+	sock->pvc.sap_addr.itf = get_itf(0);
 	sock->pvc.sap_addr.vpi = 0;
 	sock->pvc.sap_addr.vci = assign_vci++;
     }
@@ -93,6 +95,7 @@ static int send_connect(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_CONNECT);
     q_assign(&dsc,QF_call_ref,sock->call_ref);
+    if (sock->ep_ref >= 0) q_assign(&dsc,QF_ep_ref,sock->ep_ref);
     /* @@@ lots of data */
     if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
     return 0;
@@ -131,8 +134,8 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    if (!*msg->svc.sas_addr.pub && !*msg->svc.sas_addr.prv)
 		if (local_addr[0].state != ls_same)
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
-		else send_kernel(msg->vcc,0,as_okay,0,NULL,&local_addr[0].addr,
-		      NULL);
+		else send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,
+		      &local_addr[0].addr);
 	    else {
 		LOCAL_ADDR *walk;
 
