@@ -50,6 +50,7 @@ static void send_call_proceeding(SOCKET *sock)
 	sock->pvc.sap_addr.vpi = 0;
 	sock->pvc.sap_addr.vci = assign_vci++;
     }
+    if (sock->ep_ref >= 0) q_assign(&dsc,QF_ep_ref,sock->ep_ref);
     if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
@@ -74,6 +75,8 @@ static void setup_call(unsigned long call_ref)
     this->state = ss_indicated;
     this->q2931_state = qs_in_proc;
     this->call_ref = call_ref;
+    if (q_present(&in_dsc,QF_ep_ref))
+	this->ep_ref = q_fetch(&in_dsc,QF_ep_ref);
     send_call_proceeding(this);
     this->local = new_sap;
     /* if (sock->local) *this->local->sas_addr = sock->local->sas_addr; ??? */
@@ -124,6 +127,10 @@ static void send_status(SOCKET *sock,unsigned char cause,...)
     if (sock) {
 	q_assign(&dsc,QF_call_ref,sock->call_ref);
 	q_assign(&dsc,QF_call_state,(int) sock->q2931_state);
+	if (sock->ep_ref >= 0) {
+	    q_assign(&dsc,QF_ep_ref,sock->ep_ref);
+	    q_assign(&dsc,QF_ep_state,eps_map[sock->q2931_state]);
+	}
     }
     else {
 	q_assign(&dsc,QF_call_ref,0);
@@ -160,6 +167,7 @@ static void send_status_enq(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_STATUS_ENQ);
     q_assign(&dsc,QF_call_ref,sock->call_ref);
+    if (sock->ep_ref >= 0) q_assign(&dsc,QF_ep_ref,sock->ep_ref);
     if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
     /* @@@ should start T322 */
 }
@@ -195,6 +203,21 @@ static void send_restart_ack(int vpi,int vci)
 }
 
 
+static void send_drop_party_ack(unsigned long call_ref,short ep_ref,
+  unsigned char cause)
+{
+    Q_DSC dsc;
+    int size;
+
+    q_create(&dsc,q_buffer,MAX_Q_MSG);
+    q_assign(&dsc,QF_msg_type,QMSG_DROP_PARTY_ACK);
+    q_assign(&dsc,QF_call_ref,call_ref);
+    q_assign(&dsc,QF_ep_ref,ep_ref);
+    q_assign(&dsc,QF_cause,cause);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
+}
+
+
 static void q2931_call(SOCKET *sock,unsigned char mid)
 {
     int error;
@@ -447,13 +470,12 @@ void saal_okay(void)
 }
 
 
-void to_q2931(void *msg,int size)
+static void process_q2931(void *msg)
 {
     SOCKET *curr;
     unsigned long call_ref;
     unsigned char mid;
 
-    q_open(&in_dsc,msg,size);
     call_ref = q_fetch(&in_dsc,QF_call_ref)^0x800000;
     mid = q_fetch(&in_dsc,QF_msg_type);
     if (mid == QMSG_REST_ACK) return;
@@ -506,6 +528,13 @@ void to_q2931(void *msg,int size)
 	if (!curr) setup_call(call_ref);
 	return;
     }
+    if (curr && q_present(&in_dsc,QF_ep_ref) && mid != QMSG_ADD_PARTY &&
+      mid != QMSG_DROP_PARTY_ACK)
+	if (curr->ep_ref != q_fetch(&in_dsc,QF_ep_ref)) {
+	    send_drop_party_ack(call_ref,q_fetch(&in_dsc,QF_ep_ref),
+	      ATM_CV_INV_EPR);
+	    return;
+	}
     if (!curr || curr->q2931_state == qs_null) {
 	if (mid == QMSG_REL_COMP)
 	    send_release_complete(call_ref,ATM_CV_INV_CR);
@@ -515,6 +544,13 @@ void to_q2931(void *msg,int size)
 	return;
     }
     q2931_call(curr,mid);
+}
+
+
+void to_q2931(void *msg,int size)
+{
+    q_open(&in_dsc,msg,size);
+    process_q2931(msg);
     if (q_close(&in_dsc) < 0)
 	diag(COMPONENT,DIAG_ERROR,"q_close returned <0 in to_q2931");
 }
