@@ -7,6 +7,7 @@
 #include <string.h>
 #include <linux/atm.h>
 #include <linux/atmdev.h>
+#include <linux/atmsvc.h>
 
 #include "atm.h"
 #include "atmd.h"
@@ -189,34 +190,48 @@ int sap_compat(const struct sockaddr_atmsvc *old,
 }
 
 
-void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
+int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 {
     struct atm_blli *blli;
+    int pcr;
 
-    if (*sap->sas_addr.prv) /* improve @@@ */
-	q_write(dsc,QF_cdpn_esa,(void *) sap->sas_addr.prv,ATM_ESA_LEN);
+    if (!*sap->sas_addr.prv) return -EDESTADDRREQ;
+    else q_write(dsc,QF_cdpn_esa,(void *) sap->sas_addr.prv,ATM_ESA_LEN);
+	/* improve @@@ */
     switch (sap->sas_txtp.class) {
 	case ATM_NONE:
+	    if (sap->sas_rxtp.class == ATM_CBR) q_assign(dsc,QF_fw_pcr_01,0);
 	    break;
 	case ATM_UBR:
+	    diag(COMPONENT,DIAG_DEBUG,"UBR");
 	    q_assign(dsc,QF_fw_pcr_01,ATM_OC3_PCR);
 	    q_assign(dsc,QF_bw_pcr_01,ATM_OC3_PCR);
 	    q_assign(dsc,QF_best_effort,0);
 	    break;
 	case ATM_CBR:
-	    q_assign(dsc,QF_fw_pcr_01,sap->sas_txtp.min_pcr);
+	    /* here's a bit of policy: send the highest value we have */
+	    pcr = SELECT_TOP_PCR(sap->sas_txtp);
+	    diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d (%d..%d)",pcr,
+	      sap->sas_txtp.min_pcr,sap->sas_txtp.max_pcr);
+	    q_assign(dsc,QF_fw_pcr_01,pcr);
 	    break;
         default:
-	    diag(COMPONENT,DIAG_FATAL,"bad TX class (%d)",sap->sas_txtp.class);
+	    diag(COMPONENT,DIAG_ERROR,"bad TX class (%d)",sap->sas_txtp.class);
+	    return -EINVAL;
     }
     switch (sap->sas_txtp.class == ATM_UBR ? ATM_NONE : sap->sas_rxtp.class) {
 	case ATM_NONE:
+	    if (sap->sas_txtp.class == ATM_CBR) q_assign(dsc,QF_bw_pcr_01,0);
 	    break;
 	case ATM_CBR:
-	    q_assign(dsc,QF_bw_pcr_01,sap->sas_rxtp.min_pcr);
+	    pcr = SELECT_TOP_PCR(sap->sas_rxtp);
+	    diag(COMPONENT,DIAG_DEBUG,"CBR bwd %d (%d..%d)",pcr,
+	      sap->sas_rxtp.min_pcr,sap->sas_rxtp.max_pcr);
+	    q_assign(dsc,QF_bw_pcr_01,pcr);
 	    break;
         default:
-	    diag(COMPONENT,DIAG_FATAL,"bad RX class (%d)",sap->sas_txtp.class);
+	    diag(COMPONENT,DIAG_ERROR,"bad RX class (%d)",sap->sas_txtp.class);
+	    return -EINVAL;
     }
     if (sap->sas_txtp.max_sdu)
 	q_assign(dsc,QF_fw_max_sdu,sap->sas_txtp.max_sdu);
@@ -225,7 +240,7 @@ void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
     /* @@@ bearer class ? */
     /* @@@ QOS class ? */
     if (sap->sas_addr.bhli.hl_type != ATM_HL_NONE) {
-        q_assign(dsc,QF_hli_type,sap->sas_addr.bhli.hl_type);
+        q_assign(dsc,QF_hli_type,sap->sas_addr.bhli.hl_type-1);
         switch (sap->sas_addr.bhli.hl_type) {
             case ATM_HL_ISO:
                 q_write(dsc,QF_iso_hli,sap->sas_addr.bhli.hl_info,
@@ -237,15 +252,17 @@ void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
                 break;
 #ifdef UNI30
             case ATM_HL_HLP:
-                q_write(dsc,QF_iso_hli,sap->sas_addr.bhli.hl_info,4);
+                q_write(dsc,QF_hlp,sap->sas_addr.bhli.hl_info,4);
                 break;
 #endif
             case ATM_HL_VENDOR:
-                q_write(dsc,QF_iso_hli,sap->sas_addr.bhli.hl_info,7);
+		q_write(dsc,QF_hli_oui,sap->sas_addr.bhli.hl_info,3);
+		q_write(dsc,QF_app_id, sap->sas_addr.bhli.hl_info+3,4);
                 break;
             default:
-                diag(COMPONENT,DIAG_FATAL,"bad hl_type (%d)",
+                diag(COMPONENT,DIAG_ERROR,"bad hl_type (%d)",
 		  sap->sas_addr.bhli.hl_type);
+		return -EINVAL;
         }
     }
     for (blli = sap->sas_addr.blli; blli; blli = blli->next) {
@@ -296,12 +313,14 @@ void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 		    q_assign(dsc,QF_user_l3,blli->l3.user);
 		    break;
 		default:
-		    diag(COMPONENT,DIAG_FATAL,"bad l3_proto (%d)",
+		    diag(COMPONENT,DIAG_ERROR,"bad l3_proto (%d)",
 		      blli->l3_proto);
+		    return -EINVAL;
 	    }
 	}
 	break; /* @@@ allow for repeated BLLI */
     }
+    return 0;
 }
 
 
@@ -312,7 +331,7 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
 
     sap = alloc(sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
     blli = (struct atm_blli *) (sap+1);
-    memset(sap,0,sizeof(struct sockaddr_atmsvc));
+    memset(sap,0,sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
     sap->sas_family = AF_ATMSVC;
     if (q_present(dsc,QF_cdpn_esa)) /* @@@ handle E.164 too */
 	(void) q_read(dsc,QF_cdpn_esa,(void *) &sap->sas_addr.prv,ATM_ESA_LEN);
@@ -320,24 +339,35 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
 	if (q_fetch(dsc,QF_aal_type) != 5)
 	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
 	      q_fetch(dsc,QF_aal_type));
-    if (q_present(dsc,QF_best_effort))
+    if (q_present(dsc,QF_best_effort)) {
 	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_UBR;
+	diag(COMPONENT,DIAG_DEBUG,"UBR");
+    }
     else {
 	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_CBR;
-	if (q_present(dsc,QF_fw_pcr_01))
-	    sap->sas_txtp.min_pcr = sap->sas_txtp.max_pcr =
-	      q_fetch(dsc,QF_fw_pcr_01);
-	if (q_present(dsc,QF_bw_pcr_01))
-	    sap->sas_rxtp.min_pcr = sap->sas_rxtp.max_pcr =
-	      q_fetch(dsc,QF_bw_pcr_01);
+	sap->sas_txtp.max_pcr = sap->sas_rxtp.max_pcr = 0;
+	/* unbalanced decoding - always sets upper bound */
+	if (q_present(dsc,QF_fw_pcr_01)) {
+	    sap->sas_rxtp.min_pcr = 0;
+	    sap->sas_rxtp.max_pcr = q_fetch(dsc,QF_fw_pcr_01);
+	}
+	if (q_present(dsc,QF_bw_pcr_01)) {
+	    sap->sas_txtp.min_pcr = 0;
+	    sap->sas_txtp.max_pcr = q_fetch(dsc,QF_bw_pcr_01);
+	}
+	if (!sap->sas_txtp.max_pcr) sap->sas_txtp.class = ATM_NONE;
+	if (!sap->sas_rxtp.max_pcr) sap->sas_rxtp.class = ATM_NONE;
+	diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d..%d bwd %d..%d",
+	  sap->sas_rxtp.min_pcr,sap->sas_rxtp.max_pcr,sap->sas_txtp.min_pcr,
+	  sap->sas_txtp.max_pcr);
 	/* SHOULD ... fail call if anything is missing ... @@@ */
     }
-    if (q_present(dsc,QF_fw_max_sdu))
-	sap->sas_txtp.max_sdu = q_fetch(dsc,QF_fw_max_sdu);
     if (q_present(dsc,QF_bw_max_sdu))
-	sap->sas_rxtp.max_sdu = q_fetch(dsc,QF_bw_max_sdu);
+	sap->sas_txtp.max_sdu = q_fetch(dsc,QF_bw_max_sdu);
+    if (q_present(dsc,QF_fw_max_sdu))
+	sap->sas_rxtp.max_sdu = q_fetch(dsc,QF_fw_max_sdu);
     if (q_present(dsc,QG_bhli)) {
-	sap->sas_addr.bhli.hl_type = q_fetch(dsc,QF_hli_type);
+	sap->sas_addr.bhli.hl_type = q_fetch(dsc,QF_hli_type)+1;
 	switch (sap->sas_addr.bhli.hl_type) {
 	    case ATM_HL_ISO:
 		sap->sas_addr.bhli.hl_length = q_length(dsc,QF_iso_hli);
