@@ -270,7 +270,7 @@ int no_check = 0;
 	else if (!pcr) ((type *) &satm)->fld##_txtp.class = ATM_UBR; \
 	    else { \
 		    ((type *) &satm)->fld##_txtp.class = ATM_CBR; \
-		    ((type *) &satm)->fld##_txtp.min_pcr = pcr; \
+		    ((type *) &satm)->fld##_txtp.max_pcr = pcr; \
 	    } \
 	((type *) &satm)->fld##_rxtp.max_sdu = \
 	  ((type *) &satm)->fld##_txtp.max_sdu = buflen
@@ -434,8 +434,9 @@ int no_check = 0;
 				&peerlen) < 0) {
 		    err("getpeername");
 		}
-		fprintf(stderr,"ttcp-r: accept from %s\n",
-			inet_ntoa(peer.sin_addr));
+		if (atm) fprintf(stderr,"ttcp-r: accept\n");
+		else fprintf(stderr,"ttcp-r: accept from %s\n",
+			    inet_ntoa(peer.sin_addr));
 	    }
 	    
 	    if (options)  {
